@@ -1270,6 +1270,9 @@ LoadGDALImages( GDALDatasetH hDS, int band_numbers[4], int band_count,
         if( bGotNoData && pafRawData[i] == fNoDataValue )
           continue;
 
+        if( CPLIsNan(pafRawData[i]) )
+          continue;
+
         if( !bMinMaxSet ) {
           dfScaleMin = dfScaleMax = pafRawData[i];
           bMinMaxSet = TRUE;
@@ -1764,6 +1767,9 @@ msDrawRasterLayerGDAL_16BitClassification(
     if( bGotNoData && pafRawData[i] == fNoDataValue )
       continue;
 
+    if( CPLIsNan(pafRawData[i]) )
+      continue;
+
     if( !bGotFirstValue ) {
       fDataMin = fDataMax = pafRawData[i];
       bGotFirstValue = TRUE;
@@ -1915,6 +1921,9 @@ msDrawRasterLayerGDAL_16BitClassification(
         continue;
       }
 
+      if( CPLIsNan(fRawValue) )
+        continue;
+
       if(SKIP_MASK(j,i))
         continue;
 
