@@ -67,7 +67,7 @@ int msGraticuleLayerInitItemInfo(layerObj *layer);
  */
 int msGraticuleLayerOpen(layerObj *layer)
 {
-  graticuleObj *pInfo = (graticuleObj *) layer->layerinfo;
+  graticuleObj *pInfo = layer->grid;
 
   if( pInfo == NULL )
     return MS_FAILURE;
@@ -115,7 +115,7 @@ int msGraticuleLayerOpen(layerObj *layer)
  */
 int msGraticuleLayerIsOpen(layerObj *layer)
 {
-  if(layer->layerinfo)
+  if(layer->grid)
     return MS_TRUE;
 
   return MS_FALSE;
@@ -127,26 +127,6 @@ int msGraticuleLayerIsOpen(layerObj *layer)
  */
 int msGraticuleLayerClose(layerObj *layer)
 {
-  graticuleObj *pInfo = (graticuleObj *) layer->layerinfo;
-
-  if( pInfo->labelformat ) {
-    free( pInfo->labelformat );
-    pInfo->labelformat = NULL;
-  }
-
-  if( pInfo->pboundingpoints ) {
-    free( pInfo->pboundingpoints );
-    pInfo->pboundingpoints = NULL;
-  }
-
-  if( pInfo->pboundinglines ) {
-    free( pInfo->pboundinglines );
-    pInfo->pboundinglines = NULL;
-  }
-
-  free(layer->layerinfo);
-  layer->layerinfo=NULL;
-
   return MS_SUCCESS;
 }
 
@@ -155,7 +135,7 @@ int msGraticuleLayerClose(layerObj *layer)
  */
 int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
 {
-  graticuleObj *pInfo = (graticuleObj *) layer->layerinfo;
+  graticuleObj *pInfo = layer->grid;
   int iAxisTickCount = 0;
   rectObj rectMapCoordinates;
 
@@ -207,7 +187,9 @@ int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
    * These lines will be used when generating labels to get correct placement at arc/rect edge intersections.
    */
   rectMapCoordinates = layer->map->extent;
+  msFree(pInfo->pboundinglines);
   pInfo->pboundinglines   = (lineObj *)  msSmallMalloc( sizeof( lineObj )  * 4 );
+  msFree(pInfo->pboundingpoints);
   pInfo->pboundingpoints = (pointObj *) msSmallMalloc( sizeof( pointObj ) * 8 );
 
   {
@@ -223,10 +205,9 @@ int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
     pInfo->pboundinglines[0].point[1].y = rectMapCoordinates.maxy;
 
 #ifdef USE_PROJ
-    if(layer->project && msProjectionsDiffer(&(layer->projection), &(layer->map->projection)))
+    layer->project = msProjectionsDiffer(&(layer->projection), &(layer->map->projection));
+    if(layer->project)
       msProjectLine(&layer->map->projection, &layer->projection, &pInfo->pboundinglines[0]);
-    else
-      layer->project = MS_FALSE;
 #endif
 
     /*
@@ -240,10 +221,8 @@ int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
     pInfo->pboundinglines[1].point[1].y = rectMapCoordinates.miny;
 
 #ifdef USE_PROJ
-    if(layer->project && msProjectionsDiffer(&(layer->projection), &(layer->map->projection)))
+    if(layer->project)
       msProjectLine(&layer->map->projection, &layer->projection, &pInfo->pboundinglines[1]);
-    else
-      layer->project = MS_FALSE;
 #endif
 
     /*
@@ -257,10 +236,8 @@ int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
     pInfo->pboundinglines[2].point[1].y   = rectMapCoordinates.maxy;
 
 #ifdef USE_PROJ
-    if(layer->project && msProjectionsDiffer(&(layer->projection), &(layer->map->projection)))
+    if(layer->project)
       msProjectLine(&layer->map->projection, &layer->projection, &pInfo->pboundinglines[2]);
-    else
-      layer->project = MS_FALSE;
 #endif
 
     /*
@@ -274,10 +251,8 @@ int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
     pInfo->pboundinglines[3].point[1].y = rectMapCoordinates.maxy;
 
 #ifdef USE_PROJ
-    if(layer->project && msProjectionsDiffer(&(layer->projection), &(layer->map->projection)))
+    if(layer->project)
       msProjectLine(&layer->map->projection, &layer->projection, &pInfo->pboundinglines[3]);
-    else
-      layer->project = MS_FALSE;
 #endif
   }
 
@@ -289,7 +264,7 @@ int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
  */
 int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
 {
-  graticuleObj *pInfo = (graticuleObj *) layer->layerinfo;
+  graticuleObj *pInfo = layer->grid;
 
   if( pInfo->minsubdivides <= 0.0 || pInfo->maxsubdivides <= 0.0 )
     pInfo->minsubdivides = pInfo->maxsubdivides = MAPGRATICULE_ARC_SUBDIVISION_DEFAULT;
@@ -535,7 +510,7 @@ int msGraticuleLayerGetShape(layerObj *layer, shapeObj *shape, resultObj *record
  */
 int msGraticuleLayerGetExtent(layerObj *layer, rectObj *extent)
 {
-  graticuleObj *pInfo = (graticuleObj  *) layer->layerinfo;
+  graticuleObj *pInfo = layer->grid;
 
   if(pInfo) {
     *extent = pInfo->extent;
@@ -638,7 +613,7 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
   if (layer->connectiontype != MS_GRATICULE)
     return NULL;
 
-  pInfo  = (graticuleObj *) layer->layerinfo;
+  pInfo  = layer->grid;
 
   /*set cellsize if bnot already set*/
   if (map->cellsize == 0)
@@ -661,7 +636,9 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
     msProjectRect(&map->projection, &layer->projection, &searchrect); /* project the searchrect to source coords */
 #endif
 
-  msLayerOpen(layer);
+ status =  msLayerOpen(layer);
+ if(status != MS_SUCCESS)
+   return NULL;
 
   status = msLayerWhichShapes(layer, searchrect, MS_FALSE);
   if(status == MS_DONE) { /* no overlap */
@@ -696,7 +673,7 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
     msCopyShape(&shapegrid, &tmpshape);
     /* status = msDrawShape(map, layer, &tmpshape, image, -1); */
 
-    if(layer->project && msProjectionsDiffer(&(layer->projection), &(map->projection)))
+    if(layer->project)
       msProjectShape(&layer->projection, &map->projection, &shapegrid);
 
     msClipPolylineRect(&shapegrid, cliprect);
@@ -783,8 +760,10 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
               if (psValues->pasBottom[i].x == oFirstPoint.x)
                 break;
             }
-            if (i  < psValues->nBottom)
+            if (i  < psValues->nBottom) {
+              msFree(pszLabel);
               continue;
+            }
           }
           if (psValues->pasBottom == NULL) {
             psValues->pasBottom = (pointObj *)msSmallMalloc(sizeof(pointObj));
@@ -798,9 +777,7 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
 
           psValues->pasBottom[psValues->nBottom-1].x = oFirstPoint.x;
           psValues->pasBottom[psValues->nBottom-1].y = oFirstPoint.y;
-          psValues->papszBottomLabels[psValues->nBottom-1] = msStrdup(pszLabel);
-
-          msFree(pszLabel);
+          psValues->papszBottomLabels[psValues->nBottom-1] = pszLabel;
 
         }
         /*first point should cross the TOP base where y==0*/
@@ -831,8 +808,10 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
                   strcasecmp(pszLabel, psValues->papszTopLabels[i]) == 0)
                 break;
             }
-            if (i < psValues->nTop)
+            if (i < psValues->nTop) {
+              msFree(pszLabel);
               continue;
+            }
           }
 
 
@@ -848,9 +827,7 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
 
           psValues->pasTop[psValues->nTop-1].x = oLastPoint.x;
           psValues->pasTop[psValues->nTop-1].y = oLastPoint.y;
-          psValues->papszTopLabels[psValues->nTop-1] = msStrdup(pszLabel);
-
-          msFree(pszLabel);
+          psValues->papszTopLabels[psValues->nTop-1] = pszLabel;
         }
       } else { /*horzontal*/
         /*Normally lines are drawn from left to right but not always for some reason, so
@@ -893,8 +870,10 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
               if (psValues->pasLeft[i].y == oFirstPoint.y)
                 break;
             }
-            if (i < psValues->nLeft)
+            if (i < psValues->nLeft) {
+              msFree(pszLabel);
               continue;
+            }
           }
           if (psValues->pasLeft == NULL) {
             psValues->pasLeft = (pointObj *)msSmallMalloc(sizeof(pointObj));
@@ -908,8 +887,7 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
 
           psValues->pasLeft[psValues->nLeft-1].x = oFirstPoint.x;
           psValues->pasLeft[psValues->nLeft-1].y = oFirstPoint.y;
-          psValues->papszLeftLabels[psValues->nLeft-1] = msStrdup(pszLabel);
-          msFree(pszLabel);
+          psValues->papszLeftLabels[psValues->nLeft-1] = pszLabel; /* takes ownership of allocated pszLabel */
         }
         /*first point should cross the RIGHT base where x=map=>width axis*/
         if (abs((int)oLastPoint.x - map->width) <=1) {
@@ -938,8 +916,10 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
               if (psValues->pasRight[i].y == oLastPoint.y)
                 break;
             }
-            if (i < psValues->nRight)
+            if (i < psValues->nRight) {
+              msFree(pszLabel);
               continue;
+            }
           }
           if (psValues->pasRight == NULL) {
             psValues->pasRight = (pointObj *)msSmallMalloc(sizeof(pointObj));
@@ -953,9 +933,7 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
 
           psValues->pasRight[psValues->nRight-1].x = oLastPoint.x;
           psValues->pasRight[psValues->nRight-1].y = oLastPoint.y;
-          psValues->papszRightLabels[psValues->nRight-1] = msStrdup(pszLabel);
-
-          msFree(pszLabel);
+          psValues->papszRightLabels[psValues->nRight-1] = pszLabel;
         }
       }
       msFreeShape(&shapegrid);
@@ -1006,7 +984,7 @@ int msGraticuleLayerInitializeVirtualTable(layerObj *layer)
  */
 static void _FormatLabel( layerObj *pLayer, shapeObj *pShape, double dDataToFormat )
 {
-  graticuleObj *pInfo = (graticuleObj  *) pLayer->layerinfo;
+  graticuleObj *pInfo = pLayer->grid;
   char cBuffer[32];
   int iDegrees, iMinutes;
 
@@ -1041,33 +1019,43 @@ static void _FormatLabel( layerObj *pLayer, shapeObj *pShape, double dDataToForm
  */
 static int _AdjustLabelPosition( layerObj *pLayer, shapeObj *pShape, msGraticulePosition ePosition )
 {
-  graticuleObj *pInfo = (graticuleObj  *) pLayer->layerinfo;
+  graticuleObj *pInfo = pLayer->grid;
   rectObj rectLabel;
   pointObj ptPoint;
   double size = -1;
+  char *labeltxt;
 
   if( pInfo == NULL || pShape == NULL ) {
-    msSetError(MS_MISCERR, "Assertion failed: Null shape or layerinfo!, ", "_AdjustLabelPosition()");
+    msSetError(MS_MISCERR, "Assertion failed: Null shape or non-configured grid!, ", "_AdjustLabelPosition()");
     return MS_FAILURE;
   }
 
+  assert(pLayer->class[0]->numlabels >= 1);
+
   if(msCheckParentPointer(pLayer->map,"map")==MS_FAILURE)
     return MS_FAILURE;
 
   ptPoint = pShape->line->point[0];
 
 #ifdef USE_PROJ
-  if(pLayer->project && msProjectionsDiffer( &pLayer->projection, &pLayer->map->projection ))
+  if(pLayer->project)
     msProjectShape( &pLayer->projection, &pLayer->map->projection, pShape );
 #endif
 
   if(pLayer->transform) {
     msTransformShapeToPixelRound(pShape, pLayer->map->extent, pLayer->map->cellsize);
   }
-
-  size = pLayer->class[0]->labels[0]->size; /* TODO TBT: adjust minsize/maxsize/resolution. TODO RFC 77: ok to use first label? */
-  if(msGetLabelSize(pLayer->map, pLayer->class[0]->labels[0], pShape->text, size, &rectLabel, NULL) != MS_SUCCESS)
+  
+  size = pLayer->class[0]->labels[0]->size; /* TODO someday: adjust minsize/maxsize/resolution*/
+  /* We only use the first label as there's no use (yet) in defining multiple lables for GRID layers,
+   as the only label to represent is the longitude or latitude */
+  labeltxt = msShapeGetLabelAnnotation(pLayer, pShape, pLayer->class[0]->labels[0]);
+  assert(labeltxt);
+  if(msGetStringSize(pLayer->map, pLayer->class[0]->labels[0], size, labeltxt, &rectLabel) != MS_SUCCESS) {
+    free(labeltxt);
     return MS_FAILURE;  /* msSetError already called */
+  }
+  free(labeltxt);
 
   switch( ePosition ) {
     case posBottom:
@@ -1092,7 +1080,7 @@ static int _AdjustLabelPosition( layerObj *pLayer, shapeObj *pShape, msGraticule
     msTransformPixelToShape( pShape, pLayer->map->extent, pLayer->map->cellsize );
 
 #ifdef USE_PROJ
-  if(pLayer->project && msProjectionsDiffer( &pLayer->map->projection, &pLayer->projection ))
+  if(pLayer->project)
     msProjectShape( &pLayer->map->projection, &pLayer->projection, pShape );
 #endif
 
