@@ -271,33 +271,33 @@ PHP_METHOD(layerObj, __get)
   php_layer = (php_layer_object *) zend_object_store_get_object(zobj TSRMLS_CC);
 
   IF_GET_LONG("numclasses",  php_layer->layer->numclasses)
-  else IF_GET_LONG("index", php_layer->layer->index)
-    else IF_GET_LONG("status", php_layer->layer->status)
-      else IF_GET_LONG("debug",  php_layer->layer->debug)
-        else IF_GET_STRING("bandsitem", php_layer->layer->bandsitem)
-          else IF_GET_STRING("classitem", php_layer->layer->classitem)
-            else IF_GET_STRING("classgroup", php_layer->layer->classgroup)
-              else IF_GET_STRING("name", php_layer->layer->name)
-                else IF_GET_STRING("group", php_layer->layer->group)
-                  else IF_GET_STRING("data", php_layer->layer->data)
-                    else IF_GET_LONG("type",  php_layer->layer->type)
-                      else IF_GET_LONG("dump", php_layer->layer->dump)
-                        else IF_GET_DOUBLE("tolerance", php_layer->layer->tolerance)
-                          else IF_GET_LONG("toleranceunits", php_layer->layer->toleranceunits)
-                            else IF_GET_LONG("sizeunits", php_layer->layer->sizeunits)
-                              else IF_GET_DOUBLE("symbolscaledenom", php_layer->layer->symbolscaledenom)
-                                else IF_GET_LONG("maxclasses",  php_layer->layer->maxclasses)
-                                  else IF_GET_DOUBLE("minscaledenom", php_layer->layer->minscaledenom)
-                                    else IF_GET_DOUBLE("maxscaledenom", php_layer->layer->maxscaledenom)
-                                      else IF_GET_DOUBLE("labelminscaledenom", php_layer->layer->labelminscaledenom)
-                                        else IF_GET_DOUBLE("labelmaxscaledenom", php_layer->layer->labelmaxscaledenom)
-                                          else IF_GET_DOUBLE("maxgeowidth", php_layer->layer->maxgeowidth)
-                                            else IF_GET_DOUBLE("mingeowidth", php_layer->layer->mingeowidth)
-                                              else IF_GET_STRING("mask", php_layer->layer->mask)
-                                                else IF_GET_LONG("minfeaturesize", php_layer->layer->minfeaturesize)
-                                                  else IF_GET_LONG("maxfeatures", php_layer->layer->maxfeatures)
-                                                    else IF_GET_LONG("startindex", php_layer->layer->startindex)
-                                                      else IF_GET_LONG("annotate", php_layer->layer->annotate)
+  else IF_GET_STRING("encoding", php_layer->layer->encoding)
+    else IF_GET_LONG("index", php_layer->layer->index)
+      else IF_GET_LONG("status", php_layer->layer->status)
+        else IF_GET_LONG("debug",  php_layer->layer->debug)
+          else IF_GET_STRING("bandsitem", php_layer->layer->bandsitem)
+            else IF_GET_STRING("classitem", php_layer->layer->classitem)
+              else IF_GET_STRING("classgroup", php_layer->layer->classgroup)
+                else IF_GET_STRING("name", php_layer->layer->name)
+                  else IF_GET_STRING("group", php_layer->layer->group)
+                    else IF_GET_STRING("data", php_layer->layer->data)
+                      else IF_GET_LONG("type",  php_layer->layer->type)
+                        else IF_GET_LONG("dump", php_layer->layer->dump)
+                          else IF_GET_DOUBLE("tolerance", php_layer->layer->tolerance)
+                            else IF_GET_LONG("toleranceunits", php_layer->layer->toleranceunits)
+                              else IF_GET_LONG("sizeunits", php_layer->layer->sizeunits)
+                                else IF_GET_DOUBLE("symbolscaledenom", php_layer->layer->symbolscaledenom)
+                                  else IF_GET_LONG("maxclasses",  php_layer->layer->maxclasses)
+                                    else IF_GET_DOUBLE("minscaledenom", php_layer->layer->minscaledenom)
+                                      else IF_GET_DOUBLE("maxscaledenom", php_layer->layer->maxscaledenom)
+                                        else IF_GET_DOUBLE("labelminscaledenom", php_layer->layer->labelminscaledenom)
+                                          else IF_GET_DOUBLE("labelmaxscaledenom", php_layer->layer->labelmaxscaledenom)
+                                            else IF_GET_DOUBLE("maxgeowidth", php_layer->layer->maxgeowidth)
+                                              else IF_GET_DOUBLE("mingeowidth", php_layer->layer->mingeowidth)
+                                                else IF_GET_STRING("mask", php_layer->layer->mask)
+                                                  else IF_GET_LONG("minfeaturesize", php_layer->layer->minfeaturesize)
+                                                    else IF_GET_LONG("maxfeatures", php_layer->layer->maxfeatures)
+                                                      else IF_GET_LONG("startindex", php_layer->layer->startindex)
                                                         else IF_GET_LONG("transform", php_layer->layer->transform)
                                                           else IF_GET_LONG("labelcache", php_layer->layer->labelcache)
                                                             else IF_GET_LONG("postlabelcache", php_layer->layer->postlabelcache)
@@ -310,7 +310,7 @@ PHP_METHOD(layerObj, __get)
                                                                           else IF_GET_LONG("connectiontype", php_layer->layer->connectiontype)
                                                                             else IF_GET_STRING("filteritem", php_layer->layer->filteritem)
                                                                               else IF_GET_STRING("template", php_layer->layer->template)
-                                                                                else IF_GET_LONG("opacity", php_layer->layer->opacity)
+                                                                                else IF_GET_LONG("opacity", (php_layer->layer->compositer?php_layer->layer->compositer->opacity:100))
                                                                                   else IF_GET_STRING("styleitem", php_layer->layer->styleitem)
                                                                                     else IF_GET_LONG("numitems", php_layer->layer->numitems)
                                                                                       else IF_GET_LONG("numjoins", php_layer->layer->numjoins)
@@ -351,32 +351,34 @@ PHP_METHOD(layerObj, __set)
   if(Z_TYPE_P(value)==IS_NULL && !strcmp(property,"template")) {
     msFree(php_layer->layer->template);
     php_layer->layer->template = NULL;
+  } else if(!strcmp(property,"opacity")) {
+    msSetLayerOpacity(php_layer->layer,Z_LVAL_P(value));
   } else
   IF_SET_LONG("status", php_layer->layer->status, value)
-  else IF_SET_LONG("debug",  php_layer->layer->debug, value)
-    else IF_SET_STRING("classitem", php_layer->layer->classitem, value)
-      else IF_SET_STRING("bandsitem", php_layer->layer->bandsitem, value)
-        else IF_SET_STRING("classgroup", php_layer->layer->classgroup, value)
-          else IF_SET_STRING("name", php_layer->layer->name, value)
-            else IF_SET_STRING("group", php_layer->layer->group, value)
-              else IF_SET_STRING("data", php_layer->layer->data, value)
-                else IF_SET_LONG("type",  php_layer->layer->type, value)
-                  else IF_SET_LONG("dump", php_layer->layer->dump, value)
-                    else IF_SET_DOUBLE("tolerance", php_layer->layer->tolerance, value)
-                      else IF_SET_LONG("toleranceunits", php_layer->layer->toleranceunits, value)
-                        else IF_SET_LONG("sizeunits", php_layer->layer->sizeunits, value)
-                          else IF_SET_DOUBLE("symbolscaledenom", php_layer->layer->symbolscaledenom, value)
-                            else IF_SET_DOUBLE("minscaledenom", php_layer->layer->minscaledenom, value)
-                              else IF_SET_DOUBLE("maxscaledenom", php_layer->layer->maxscaledenom, value)
-                                else IF_SET_LONG("minfeaturesize", php_layer->layer->minfeaturesize, value)
-                                  else IF_SET_DOUBLE("labelminscaledenom", php_layer->layer->labelminscaledenom, value)
-                                    else IF_SET_DOUBLE("labelmaxscaledenom", php_layer->layer->labelmaxscaledenom, value)
-                                      else IF_SET_DOUBLE("maxgeowidth", php_layer->layer->maxgeowidth, value)
-                                        else IF_SET_DOUBLE("mingeowidth", php_layer->layer->mingeowidth, value)
-                                          else IF_SET_STRING("mask", php_layer->layer->mask, value)
-                                            else IF_SET_LONG("maxfeatures", php_layer->layer->maxfeatures, value)
-                                              else IF_SET_LONG("startindex", php_layer->layer->startindex, value)
-                                                else IF_SET_LONG("annotate", php_layer->layer->annotate, value)
+  else IF_SET_STRING("encoding",  php_layer->layer->encoding, value)
+    else IF_SET_LONG("debug",  php_layer->layer->debug, value)
+      else IF_SET_STRING("classitem", php_layer->layer->classitem, value)
+        else IF_SET_STRING("bandsitem", php_layer->layer->bandsitem, value)
+          else IF_SET_STRING("classgroup", php_layer->layer->classgroup, value)
+            else IF_SET_STRING("name", php_layer->layer->name, value)
+              else IF_SET_STRING("group", php_layer->layer->group, value)
+                else IF_SET_STRING("data", php_layer->layer->data, value)
+                  else IF_SET_LONG("type",  php_layer->layer->type, value)
+                    else IF_SET_LONG("dump", php_layer->layer->dump, value)
+                      else IF_SET_DOUBLE("tolerance", php_layer->layer->tolerance, value)
+                        else IF_SET_LONG("toleranceunits", php_layer->layer->toleranceunits, value)
+                          else IF_SET_LONG("sizeunits", php_layer->layer->sizeunits, value)
+                            else IF_SET_DOUBLE("symbolscaledenom", php_layer->layer->symbolscaledenom, value)
+                              else IF_SET_DOUBLE("minscaledenom", php_layer->layer->minscaledenom, value)
+                                else IF_SET_DOUBLE("maxscaledenom", php_layer->layer->maxscaledenom, value)
+                                  else IF_SET_LONG("minfeaturesize", php_layer->layer->minfeaturesize, value)
+                                    else IF_SET_DOUBLE("labelminscaledenom", php_layer->layer->labelminscaledenom, value)
+                                      else IF_SET_DOUBLE("labelmaxscaledenom", php_layer->layer->labelmaxscaledenom, value)
+                                        else IF_SET_DOUBLE("maxgeowidth", php_layer->layer->maxgeowidth, value)
+                                          else IF_SET_DOUBLE("mingeowidth", php_layer->layer->mingeowidth, value)
+                                            else IF_SET_STRING("mask", php_layer->layer->mask, value)
+                                              else IF_SET_LONG("maxfeatures", php_layer->layer->maxfeatures, value)
+                                                else IF_SET_LONG("startindex", php_layer->layer->startindex, value)
                                                   else IF_SET_LONG("transform", php_layer->layer->transform, value)
                                                     else IF_SET_LONG("labelcache", php_layer->layer->labelcache, value)
                                                       else IF_SET_LONG("postlabelcache", php_layer->layer->postlabelcache, value)
@@ -388,7 +390,6 @@ PHP_METHOD(layerObj, __set)
                                                                   else IF_SET_STRING("connection", php_layer->layer->connection, value)
                                                                     else IF_SET_STRING("filteritem", php_layer->layer->filteritem, value)
                                                                       else IF_SET_STRING("template", php_layer->layer->template, value)
-                                                                        else IF_SET_LONG("opacity", php_layer->layer->opacity, value)
                                                                           else IF_SET_STRING("styleitem", php_layer->layer->styleitem, value)
                                                                             else IF_SET_LONG("num_processing", php_layer->layer->numprocessing, value)
                                                                               else IF_SET_STRING("requires", php_layer->layer->requires, value)
@@ -1619,7 +1620,7 @@ PHP_METHOD(layerObj, executeWFSGetFeature)
 
 /* {{{ proto int applySLD(string sldxml, string namedlayer)
    Apply the SLD document to the layer object. The matching between the sld
-   document and the layer will be done using the layer’s name. If a
+   document and the layer will be done using the layer's name. If a
    namedlayer argument is passed (argument is optional), the NamedLayer in
    the sld that matchs it will be used to style the layer. */
 PHP_METHOD(layerObj, applySLD)
@@ -1652,7 +1653,7 @@ PHP_METHOD(layerObj, applySLD)
 /* {{{ proto int applySLDURL(string sldurl, string namedlayer)
    Apply the SLD document pointed by the URL to the layer object. The
    matching between the sld document and the layer will be done using the
-   layer’s name. If a namedlayer argument is passed (argument is optional),
+   layer's name. If a namedlayer argument is passed (argument is optional),
    the NamedLayer in the sld that matchs it will be used to style the
    layer. */
 PHP_METHOD(layerObj, applySLDURL)
